(function() {
	// Shortcuts like prototype.js
	var $ = dojo.byId;
	var $$ = dojo.query;
	
	function log(str)
	{
		document.body.appendChild(document.createTextNode(str + " " ));
	}

	/* Don't open the menu for certain well know, unprintable keys. */
	function ignoreFieldKeyEvent(e)
	{
		if (e.ctrlKey || e.altKey) return false;

		switch(e.keyCode)
		{
		case dojo.keys.ENTER:
		case dojo.keys.ESCAPE:
		case dojo.keys.LEFT_ARROW:
		case dojo.keys.RIGHT_ARROW:
		case dojo.keys.UP_ARROW:
		case dojo.keys.DOWN_ARROW:
		case dojo.keys.TAB:
		case dojo.keys.SHIFT:
		case dojo.keys.ALT:
		case dojo.keys.CAPS_LOCK:
		case dojo.keys.PAGE_DOWN:
		case dojo.keys.PAGE_UP:
		case dojo.keys.LEFT_WINDOW:
		case dojo.keys.RIGHT_WINDOW:
		case dojo.keys.HOME:
		case dojo.keys.END:
		case dojo.keys.CTRL:
		case dojo.keys.F5:
			return true;
			
		default:
			return false;
		}
	}

	/**
	 * Build a container object for the menu. Floats under the input field.
	 * @param under the text input to show the menu under.
	 */
	function newMenu(under)
	{
		var pos = dojo.coords(under);
		var menu = $$(document.createElement('div'))
			.addClass('qs-menu')
			.style({
				display: 'none',
				position: 'absolute',
				top: pos.y + under.offsetHeight + 2 + 'px',
				left: pos.x + 'px'})
			.place(document.body);
	
		return menu[0];
	}

	/** Constructor. Attaches events to text input field to
	 * support searching and CR suggestions. Note that this put into
	 * the global namespace.
	 * @param url URL of the quick search servlet that returns HTML results.
	 * @param token to hit the QS servlet with.
	 * @param field a text input that the user types into.
	 * @param options object of optional arguments. Includes:
	 *		button - optional button to make the menu appear if not always shown
	 *		menuContainer - DOM element to hold the search results. If not given, floating menu is created automatically
	 *		onStartSearch - call function for when a search starts
	 *		onEndSearch - callback function for when search completes
	 *		onSelect - callback function after a user clicks a CR results. Passed the CRID as an argument.
	 */
	window.QuickSearch = function(url, token, field, options)
	{
		this._url = url;
		this._token = token;
		this._field = field;
		this._options = options || {};

		this._menu = options.menuContainer || newMenu(field);
		// We set _alwaysVisible if we don't want to close the menu
		// in response to clicks outside of it or esc key. This is
		// used in the OSLC select dialog where the "menu" is the only
		// thing there, and there's no real reason to close it: it's
		// more like a list of results.
		this._alwaysVisible = options.menuContainer != null;
		
		// Track the event listeners that are only connected when the menu opens.
		// This lets us clear them easily when it closes.
		this._openHandles = [];

		// Each ajax call gets an ID to we don't process them out of order.
		this._ajaxCounter = 0;

		field.setAttribute('autocomplete', 'OFF');

		dojo.connect(field, 'onkeyup', this, '_fieldOnKeyUp');
		options.button && dojo.connect(options.button, 'onclick', this, '_buttonOnClick');
		dojo.connect(this._menu, 'onclick', this, '_menuOnClick');
	}

	QuickSearch.prototype._buttonOnClick = function(e)
	{
		dojo.stopEvent(e);
		this._field.focus();
		this.open();
	};

	QuickSearch.prototype._fieldOnKeyUp = function(e)
	{
		var k = e.keyCode;

		if (k == dojo.keys.DOWN_ARROW)
		{
			if (this._isOpen) return;

			dojo.stopEvent(e);
			this.open();
		}
		else if (!this._alwaysVisible &&
			k === dojo.keys.BACKSPACE &&
			this._field.value === '')
		{
			// all text deleted, so close the menu
			this.close();
			if (this.pendingOpen) window.clearTimeout(this.pendingOpen);
			return;
		}
		else if (!ignoreFieldKeyEvent(e))
		{
			// open after a delay to avoid too many ajax calls
			var self = this;
			if (this.pendingOpen) window.clearTimeout(this.pendingOpen);
			this.pendingOpen = window.setTimeout(function() {self.open()}, 400);
		}
	};
	
	/**
	 * Runs a search with the current value of the text input.
	 * Opens the menu to show the results.
	 * @param pageNumber optional page of the results to show. The first query
	 * is always 0; next/prev links can ask for a different page.
	 * Too high a number will return no results.
	 */
	QuickSearch.prototype.open = function(pageNumber)
	{
		pageNumber = pageNumber || 0;
		this._isOpen = true;

		// Increment counter to distinguish each request.
		// Only show results from requests that match the current counter
		// in case responses come out of order.
		var myAjaxCounter = ++this._ajaxCounter;
		var self = this;
		
		// Fire callback is requested.
		if (this._options.onStartSearch) this._options.onStartSearch();

		dojo.xhrGet({
			url: this._url,
			content: {
				q: dojo.trim(this._field.value),
				token: this._token,
				pageNumber: pageNumber,
				pageSize: 10},
			load: onSearchLoad,
			error: onSearchError
			});

		function onSearchLoad(results)
		{	
			if (self._options.onEndSearch) self._options.onEndSearch();

			// Make sure we're still interested in the resulted. User might
			// have moved on or request could come back out-of-order.
			if (!self._isOpen || myAjaxCounter !== self._ajaxCounter) return;

			self._menu.innerHTML = results;
			self._menu.style.display = '';
			
			// Clear event listeners to ensure we don't add them more than once.
			dojo.forEach(self._openHandles, dojo.disconnect);

			self._openHandles.push(
				dojo.connect(document, 'onkeydown', self, '_menuOnKeyDown'));
			self._openHandles.push(
				dojo.connect(self._menu, 'onmouseover', self, '_rowOnOver'));
			self._openHandles.push(
				dojo.connect(self._menu, 'onmouseout', self, '_rowOnOut'));
			
			if (!self._alwaysVisible)
			{
				self._openHandles.push(
					dojo.connect(document, 'onclick', self, 'close'));
			}
		}

		function onSearchError(error)
		{
			if (self._options.onEndSearch) self._options.onEndSearch();
			if (!self._alwaysVisible) self.close();
			if (error.message) alert(error.message);
			window.console && window.console.error && window.console.error(error.message || error);
		}
	};

	/**
	 * Closes the menu if it's open.
	 */
	QuickSearch.prototype.close = function()
	{
		this._isOpen = false;
		dojo.forEach(self._openHandles, dojo.disconnect);
		this._menu.style.display = 'none';
	};

	/** Selects a row highlighted row/CR; puts the CRID in the text input. */
	QuickSearch.prototype._select = function(row)
	{
		this._field.focus();

		if (row)
		{
			this._field.value = row.getAttribute('crid');
			var synopsis = row.getAttribute('synopsis')
			this._options.onSelect && this._options.onSelect(this._field.value, synopsis);
			this.close();
		}
		else if (!this._alwaysVisible)
		{
			this.close();
		}
	};
	
	QuickSearch.prototype._menuOnClick = function(e)
	{
		dojo.stopEvent(e);
		var clicked = e.target;
		
		// This is a normal CR.
		if (clicked.nodeName == 'LI')
		{
			this._select(clicked);
			return;
		}
		
		// Clicked 'next' or 'previous' page link
		var nextOrPrev = dojo.attr(clicked, 'next') || dojo.attr(clicked, 'previous');
		if (nextOrPrev)
		{
			this._field.focus();
			this.open(nextOrPrev);
			return;
		}
	};

	/** 
	 * Key event handler when menu is open. Mainly handles navigation through
	 * the arrow keys and 'enter' to select a CR.
	 */
	QuickSearch.prototype._menuOnKeyDown = function(e)
	{
		var c = e.keyCode;
		
		if (e.target.nodeName == 'A' &&
			(c == dojo.keys.ENTER || c == dojo.keys.SPACE))
		{
			// Don't override keyboard navigation of links (specifically next/prev links).
			return;
		}

		if (c == dojo.keys.ENTER)
		{
			dojo.stopEvent(e);
			this._select(this._lastHighlight);
		}
		else if (!this._alwaysVisible && c === dojo.keys.ESCAPE)
		{
			this.close();
		}
		else if (c === dojo.keys.DOWN_ARROW ||
			c === dojo.keys.UP_ARROW)
		{
			dojo.stopEvent(e);
			
			var rows = this._menu.getElementsByTagName('LI');
			var len = rows.length;
			if (len === 0) return;
			
			var oldI = dojo.indexOf(rows, this._lastHighlight);
			// this can be -1 if nothing was highlighted. This still works with that.
			
			if (c == dojo.keys.DOWN_ARROW)
			{
				// next one 'down', warpping to top if necessary
				var newI = (oldI + 1) % len;
			}
			else
			{
				// next one 'up'; wrapping to bottom if necessary
				var newI = oldI - 1;
				if (newI < 0) newI = len - 1;
			}
	
			this._highlight(rows[newI]);
		}
	};
	
	/** Finds first ancestor node with matching tag name. */
	function ancestor(el, tag)
	{
		var p = el.parentNode;
		while (p)
		{
			if (p.nodeName === tag.toUpperCase()) return el;
			p = p.parentNode;
		}
	}
	var i = 0;
	/** Adds visual "higlight" to a CR item (LI) in the results and
	 * saves the element in _lastHighlight. If the user
	 * clicks or hits 'enter' now, the _lastHighlight is
	 * the selected CR. Unhighligts old row if necessary.
	 */
	QuickSearch.prototype._highlight = function(row)
	{
		var div = ancestor(row, 'DIV');
		
		this._unhighlight();
		
		dojo.addClass(row, 'highlight');
		this._lastHighlight = row;
	};
	
	QuickSearch.prototype._unhighlight = function()
	{
		if (this._lastHighlight)
		{
			dojo.removeClass(this._lastHighlight, 'highlight');
			delete this._lastHighlight;
		}
	};

	QuickSearch.prototype._rowOnOver = function(e)
	{
		var row = e.target;
		if (row.nodeName == 'LI') this._highlight(row);
	};

	QuickSearch.prototype._rowOnOut = function(e)
	{
		this._unhighlight();
	};
})();
