# <trigger>
#	<usage>async:perl crAssignEmail.pl</usage>
#	<version>4.4</version>
#	<description>When a CR is assigned to a user, check to see if an e-mail should be sent. An e-mail should be sent if
#                    the user assigning the CR is not the same user to which the e-mail should be sent.
#
#                    An email should not be sent if any of the following criteria is met:
#                    1.) Trigger fails to show the CR.
#                    2.) The resolver's email address cannot be found.
#                    3.) Create task was checked, this will cause the underlying CM trigger to send an email.
#
#                    NOTE: The pt.cfg file must be edited prior to using this script.  There are two
#                          configuration entries that must be filled out, SMTP_SERVER and FROM_EMAIL_ADDRESS. The
#                          SMTP_SERVER entry is used to define the address of the SMTP server which should be used
#                          when sending e-mails.  And the FROM_EMAIL_ADDRESS entry will be used as the 'from' in
#                          all e-mails sent via Triggers.
#
#       </description>
# </trigger>

use ChangeSynergy::utf8Conditional;
use ChangeSynergy::emailHelper;
use strict;	
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
#The trigger xml document is always the last parameter.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Will die if smtp server or from address not set.
ChangeSynergy::EmailHelper::verifyMailData($trigger);

#Get all the attributes from the trigger file.
my %trigger_attributes = $trigger->get_attributes();

#Get the value of the _CREATE_TASK attribute.
my $create = $trigger_attributes{"_CREATE_TASK"};

if((defined($create)) && ("\L$create" eq "true"))
{
	#Create task was checked, underlying trigger will cause email to be sent
	exit 0;
}

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object, password is not needed, uses the token
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "", $trigger->get_role(),
		   $trigger->get_token(), $trigger->get_database());

#Setup the EmailHelper module so it can share the csapi, user and trigger variables.
ChangeSynergy::EmailHelper::setup($csapi, $user, $trigger);

#Get the crid from the trigger.
my $crid = $trigger->get_object_id();

#Get extra CR information from the server
my $crInfo = ChangeSynergy::EmailHelper::getCrDetails($crid, "problem_synopsis|crstatus|severity|release|resolver|priority|_COMMENTS");

#Get the resolver, will exit if the resolver is the same as the user who fired the script.
my $resolver = getResolver();

#Use the ChangeSynergy::EmailHelper to open the email and get things going.
ChangeSynergy::EmailHelper::openEmail(getUserEmail(), "", "", "IBM Rational Change assignment notification - CR " . $crid);

my $mainTableData = ChangeSynergy::EmailHelper::createCrInfoBlockData($crInfo, "Assigner");
ChangeSynergy::EmailHelper::sendStrippedTable("Change Request Information", $mainTableData);

my $additionalTableData = createAdditionalBlockData($crInfo);
ChangeSynergy::EmailHelper::sendStrippedTable("Additional Information", $additionalTableData);

#Closes the email out and attaches the inline image.
ChangeSynergy::EmailHelper::closeEmail();

######################## Local Methods ########################

# Get the resolver attribute from the CR info.
sub getResolver
{
	#Get the resolver attribute by its name from the problem, then get its value.
	my $tmpResolver = $crInfo->getDataObjectByName("resolver")->getValue();

	if(!defined($tmpResolver))
	{
		print "Unable to find resolver name, could not send e-mail";
		exit 0;
	}

	if($tmpResolver eq $trigger->get_user())
	{
		#The resolver is the same user that caused the script to fire, do not send e-mail
		exit 0;
	}

	return $tmpResolver;
}

# Get the users email address, if this fails an email cannot be sent to the user.
sub getUserEmail
{
	my $user_email;

	eval
	{
		$user_email = $csapi->GetUserPreference($user, $resolver, "user_email")->getResponseData();
	};

	if ($@)
	{
		print "User email address for " . $resolver . " was not defined in LDAP, cannot send an email";
		exit 0;
	}

	return $user_email;
}

# Create the second table full of information. This table holds the problem synopsis, release, severity, priority and comments.
sub createAdditionalBlockData
{
	my $crInfo = shift;

	my @loop_data = ();  # initialize an array to hold the loop data.

	my %row_data = ();
	$row_data{alias} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("_COMMENTS")->getLabel());
    
	if ($crInfo->getDataObjectByName("_COMMENTS")->getType() eq "CCM_EXTENDED_TEXT")
	{
		$row_data{value} = $trigger_attributes{"_COMMENTS"};
	}
	else
	{
		$row_data{value} = ChangeSynergy::util::htmlEncode($trigger_attributes{"_COMMENTS"});
	}
	$row_data{zebra} = 1;
	push(@loop_data, \%row_data);

	my %row_data1 = ();
	$row_data1{alias} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("priority")->getLabel());
    $row_data1{value} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("priority")->getValue());
	$row_data1{zebra} = 0;
	push(@loop_data, \%row_data1);

	my %row_data2 = ();
	$row_data2{alias} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("problem_synopsis")->getLabel());
    $row_data2{value} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("problem_synopsis")->getValue());
	$row_data2{zebra} = 1;
	push(@loop_data, \%row_data2);

	my %row_data3 = ();
	$row_data3{alias} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("release")->getLabel());
    $row_data3{value} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("release")->getValue());
	$row_data3{zebra} = 0;
	push(@loop_data, \%row_data3);

	my %row_data4 = ();
	$row_data4{alias} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("severity")->getLabel());
    $row_data4{value} = ChangeSynergy::util::htmlEncode($crInfo->getDataObjectByName("severity")->getValue());
	$row_data4{zebra} = 1;
	push(@loop_data, \%row_data4);

	return \@loop_data; 
}
