// Copyright IBM Corporation 2000, 2009

/*
<trigger>
	<usage>bsf create_associated_task.js "See CR %problem_number: %problem_synopsis"</usage>
	<version>5.0</version>
	<description>
		Associates a new task to a CR during a submit or transition. The task is only created if an attribute named _CREATE_TASK is sent as part of the transition request. Normally this is done by including a _CREATE_TASK attribute of type CCM_TOGGLE on a transition form, allowing a user to simply click a checkbox to decide if a new task is created during the transition.
	</description>
	<argument1>
		<name>synopsis</name>
		<required>no</required>
		<type>string</type>
		<description>The synopsis to give to the new task. A synopsis with spaces in it should be enclosed in double-quotes. The synopsis can refer to attributes from the parent CR by proceeding the attribute names with a percent sign (%). E.g., the default task synopsis is "See CR %problem_number: %problem_synopsis" and includes the parent problem_number and problem_synopsis. If you refer to a non-existent attribute, the attribute name is left in the synopsis as-is.
		</description>
	</argument1>
	<argument2>
		<name>-releaseattr cr_release_attribute_name</name>
		<required>no</required>
		<type>string</type>
		<description>
			Normally the 'release' attribute is copied from the CR to the new task. If specified, this lets you use
			a different attribute from the CR as the tasks release, e.g., -releaseattr target_release.
		</description>
	</argument2>
	<argument3>
		<name>-resolverattr cr_user_attribute</name>
		<required>no</required>
		<type>string</type>
		<description>
			Normally the 'resolver' attribute is copied from the CR to the new task. If specified, this lets you use
			a different attribute from the CR as the tasks resolver, e.g., -resolverattr enterer.
		</description>
	</argument3>
</trigger>
*/

importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

// check if the _CREATE_TASK pseudo-attribute is present.
// If so, we should create a new task.
var createTask = false;
for (var i = 0; i < event.attributes.length; i++)
{
	if (event.attributes[i].name == '_CREATE_TASK')
	{
		createTask = true;
		break;
	}
}

if (createTask)
{
	// Some lifecycles may want to set the task 'resolver' and 'release' from
	// custom attributes on the CR. Check for custom attribute names, or use the default if there are none.
	var options = OptionParser.parse(args);
	var resolverAttrName = options.getOption('-resolverattr') || 'resolver';
	var releaseAttrName = options.getOption('-releaseattr') || 'release';

	// use the given task synopsis (a template), or a default
	var synopsisTemplate = new KeywordString(options.args[0] || 'See CR %problem_number: %problem_synopsis');

	// get all the attribute values we need from the CR that we'll apply to the task.
	// This includes the attributes needed to expanded the attribute keywords in the synopsis.
	var crAttributes = change.collectAttributes(
		event.triggerUser.token,
		event,
		[resolverAttrName, releaseAttrName].concat(synopsisTemplate.getKeywords()));

	var taskSynopsis = synopsisTemplate.replaceKeywords(mapToStringValues(crAttributes));

	var taskAttributes = [
		new Attribute('task_synopsis', taskSynopsis),
		new Attribute('priority', 'medium')
		];
		
	// ensure the inherited attributes are on the CR before we use them.
	// They may not be required for this transition
	var crResolver = crAttributes.get(resolverAttrName);
	var crRelease = crAttributes.get(releaseAttrName);
	if (crResolver && crResolver.stringValue) taskAttributes.push(new Attribute('resolver', crResolver.stringValue));
	if (crRelease && crRelease.stringValue) taskAttributes.push(new Attribute('release', crRelease.stringValue));

	var taskId = change.createAssociatedTask(
		event.triggerUser.token,
		event.objectId,
		taskAttributes);

	log.debug('CR ' + event.objectId + ': associated new task ' + taskId);
}

/**
 * Changes Map<String, Attribute> into a Map<String, String>,
 * using the Attribute string values as the new map's values.
 * The latter form can be passed to replaceKeywords.
 */
function mapToStringValues(attributeMap)
{
	var valueMap = new java.util.HashMap();
	for (var iter = attributeMap.keySet().iterator(); iter.hasNext();)
	{
		var key = iter.next();
		valueMap.put(key, attributeMap.get(key).toString());
	}

	return valueMap;
}
